# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Iterable
from datetime import datetime
from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncCursor, AsyncCursor
from ..._base_client import AsyncPaginator, make_request_options
from ...types.lookup_tables import (
    lookup_table_revision_list_params,
    lookup_table_revision_create_params,
    lookup_table_revision_update_params,
    lookup_table_revision_update_status_params,
)
from ...types.lookup_tables.lookup_table_revision_response import LookupTableRevisionResponse

__all__ = ["LookupTableRevisionsResource", "AsyncLookupTableRevisionsResource"]


class LookupTableRevisionsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> LookupTableRevisionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return LookupTableRevisionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> LookupTableRevisionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return LookupTableRevisionsResourceWithStreamingResponse(self)

    def create(
        self,
        lookup_table_id: str,
        *,
        org_id: str | None = None,
        fields: Iterable[lookup_table_revision_create_params.Field],
        keys: SequenceNotStr[str],
        name: str,
        custom_fields: Dict[str, Union[str, float]] | Omit = omit,
        start_date: Union[str, datetime] | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LookupTableRevisionResponse:
        """
        Create a new Revision for a Lookup Table.

        Fields and Keys for Revision schema: Use the `fields` parameter to define a
        Revision schema containing up to 10 number or string fields. Use the `keys`
        parameter to specify which are the key fields:

        - At least one field must be a non-key field and at least one a key field.
        - Up to 5 key fields can be defined.
        - Using multiple key fields: ensure that the order in which they are defined
          matches the order in which you want to use them in any Lookup functions that
          reference the Revision's Lookup Table, because this is the order in which they
          will be passed into the function. The order of non-key fields is not
          constrained in this way.

        Revision status: when you first create a Lookup Table Revision it has DRAFT
        status. Use the
        [Update LookupTableRevision Status](www.m3ter.com/docs/api#tag/LookupTableRevision/operation/UpdateLookupTableRevisionStatus)
        call to change a Revision's status.

        Args:
          fields: The list of fields of the Lookup Table Revision.

          keys: The ordered keys of the Lookup Table Revision.

          name: Descriptive name for the Lookup Table Revision.

          custom_fields: User defined fields enabling you to attach custom data. The value for a custom
              field can be either a string or a number.

              If `customFields` can also be defined for this entity at the Organizational
              level, `customField` values defined at individual level override values of
              `customFields` with the same name defined at Organization level.

              See
              [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
              in the m3ter documentation for more information.

          start_date: The start date of the Lookup Table Revision.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not lookup_table_id:
            raise ValueError(f"Expected a non-empty value for `lookup_table_id` but received {lookup_table_id!r}")
        return self._post(
            f"/organizations/{org_id}/lookuptables/{lookup_table_id}/revisions",
            body=maybe_transform(
                {
                    "fields": fields,
                    "keys": keys,
                    "name": name,
                    "custom_fields": custom_fields,
                    "start_date": start_date,
                    "version": version,
                },
                lookup_table_revision_create_params.LookupTableRevisionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LookupTableRevisionResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        lookup_table_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LookupTableRevisionResponse:
        """
        Retrieve a Lookup Table Revision for the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not lookup_table_id:
            raise ValueError(f"Expected a non-empty value for `lookup_table_id` but received {lookup_table_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/organizations/{org_id}/lookuptables/{lookup_table_id}/revisions/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LookupTableRevisionResponse,
        )

    def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        lookup_table_id: str,
        fields: Iterable[lookup_table_revision_update_params.Field],
        keys: SequenceNotStr[str],
        name: str,
        custom_fields: Dict[str, Union[str, float]] | Omit = omit,
        start_date: Union[str, datetime] | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LookupTableRevisionResponse:
        """
        Update a Lookup Table Revision for the given UUID.

        **NOTE:** If you've already added data to a Lookup Table Revision - see the
        following
        [Lookup Table Revision Data](https://www.m3ter.com/docs/api#tag/LookupTableRevisionData)
        section - then you won't be able to change the Revision's field schema and
        you'll receive an error if you try do this. Create a new Revision instead, or
        delete the data items first.

        Args:
          fields: The list of fields of the Lookup Table Revision.

          keys: The ordered keys of the Lookup Table Revision.

          name: Descriptive name for the Lookup Table Revision.

          custom_fields: User defined fields enabling you to attach custom data. The value for a custom
              field can be either a string or a number.

              If `customFields` can also be defined for this entity at the Organizational
              level, `customField` values defined at individual level override values of
              `customFields` with the same name defined at Organization level.

              See
              [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
              in the m3ter documentation for more information.

          start_date: The start date of the Lookup Table Revision.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not lookup_table_id:
            raise ValueError(f"Expected a non-empty value for `lookup_table_id` but received {lookup_table_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._put(
            f"/organizations/{org_id}/lookuptables/{lookup_table_id}/revisions/{id}",
            body=maybe_transform(
                {
                    "fields": fields,
                    "keys": keys,
                    "name": name,
                    "custom_fields": custom_fields,
                    "start_date": start_date,
                    "version": version,
                },
                lookup_table_revision_update_params.LookupTableRevisionUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LookupTableRevisionResponse,
        )

    def list(
        self,
        lookup_table_id: str,
        *,
        org_id: str | None = None,
        ids: SequenceNotStr[str] | Omit = omit,
        next_token: str | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursor[LookupTableRevisionResponse]:
        """
        List LookupTableRevision entities

        Args:
          ids: List of Lookup Table Revision IDs to retrieve

          next_token: Token to supply for multi page retrievals

          page_size: Number of LookupTable to retrieve per page

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not lookup_table_id:
            raise ValueError(f"Expected a non-empty value for `lookup_table_id` but received {lookup_table_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/lookuptables/{lookup_table_id}/revisions",
            page=SyncCursor[LookupTableRevisionResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "ids": ids,
                        "next_token": next_token,
                        "page_size": page_size,
                    },
                    lookup_table_revision_list_params.LookupTableRevisionListParams,
                ),
            ),
            model=LookupTableRevisionResponse,
        )

    def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        lookup_table_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LookupTableRevisionResponse:
        """
        Delete the Lookup Table Revision for the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not lookup_table_id:
            raise ValueError(f"Expected a non-empty value for `lookup_table_id` but received {lookup_table_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/organizations/{org_id}/lookuptables/{lookup_table_id}/revisions/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LookupTableRevisionResponse,
        )

    def update_status(
        self,
        id: str,
        *,
        org_id: str | None = None,
        lookup_table_id: str,
        status: Literal["DRAFT", "PUBLISHED", "ARCHIVED"] | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LookupTableRevisionResponse:
        """
        Update the status of a Lookup Table Revision for the given UUID.

        Args:
          status: Status of a Lookup Table Revision

          version: The version of the LookupTableRevision.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not lookup_table_id:
            raise ValueError(f"Expected a non-empty value for `lookup_table_id` but received {lookup_table_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._put(
            f"/organizations/{org_id}/lookuptables/{lookup_table_id}/revisions/{id}/status",
            body=maybe_transform(
                {
                    "status": status,
                    "version": version,
                },
                lookup_table_revision_update_status_params.LookupTableRevisionUpdateStatusParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LookupTableRevisionResponse,
        )


class AsyncLookupTableRevisionsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncLookupTableRevisionsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncLookupTableRevisionsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncLookupTableRevisionsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return AsyncLookupTableRevisionsResourceWithStreamingResponse(self)

    async def create(
        self,
        lookup_table_id: str,
        *,
        org_id: str | None = None,
        fields: Iterable[lookup_table_revision_create_params.Field],
        keys: SequenceNotStr[str],
        name: str,
        custom_fields: Dict[str, Union[str, float]] | Omit = omit,
        start_date: Union[str, datetime] | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LookupTableRevisionResponse:
        """
        Create a new Revision for a Lookup Table.

        Fields and Keys for Revision schema: Use the `fields` parameter to define a
        Revision schema containing up to 10 number or string fields. Use the `keys`
        parameter to specify which are the key fields:

        - At least one field must be a non-key field and at least one a key field.
        - Up to 5 key fields can be defined.
        - Using multiple key fields: ensure that the order in which they are defined
          matches the order in which you want to use them in any Lookup functions that
          reference the Revision's Lookup Table, because this is the order in which they
          will be passed into the function. The order of non-key fields is not
          constrained in this way.

        Revision status: when you first create a Lookup Table Revision it has DRAFT
        status. Use the
        [Update LookupTableRevision Status](www.m3ter.com/docs/api#tag/LookupTableRevision/operation/UpdateLookupTableRevisionStatus)
        call to change a Revision's status.

        Args:
          fields: The list of fields of the Lookup Table Revision.

          keys: The ordered keys of the Lookup Table Revision.

          name: Descriptive name for the Lookup Table Revision.

          custom_fields: User defined fields enabling you to attach custom data. The value for a custom
              field can be either a string or a number.

              If `customFields` can also be defined for this entity at the Organizational
              level, `customField` values defined at individual level override values of
              `customFields` with the same name defined at Organization level.

              See
              [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
              in the m3ter documentation for more information.

          start_date: The start date of the Lookup Table Revision.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not lookup_table_id:
            raise ValueError(f"Expected a non-empty value for `lookup_table_id` but received {lookup_table_id!r}")
        return await self._post(
            f"/organizations/{org_id}/lookuptables/{lookup_table_id}/revisions",
            body=await async_maybe_transform(
                {
                    "fields": fields,
                    "keys": keys,
                    "name": name,
                    "custom_fields": custom_fields,
                    "start_date": start_date,
                    "version": version,
                },
                lookup_table_revision_create_params.LookupTableRevisionCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LookupTableRevisionResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        lookup_table_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LookupTableRevisionResponse:
        """
        Retrieve a Lookup Table Revision for the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not lookup_table_id:
            raise ValueError(f"Expected a non-empty value for `lookup_table_id` but received {lookup_table_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/organizations/{org_id}/lookuptables/{lookup_table_id}/revisions/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LookupTableRevisionResponse,
        )

    async def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        lookup_table_id: str,
        fields: Iterable[lookup_table_revision_update_params.Field],
        keys: SequenceNotStr[str],
        name: str,
        custom_fields: Dict[str, Union[str, float]] | Omit = omit,
        start_date: Union[str, datetime] | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LookupTableRevisionResponse:
        """
        Update a Lookup Table Revision for the given UUID.

        **NOTE:** If you've already added data to a Lookup Table Revision - see the
        following
        [Lookup Table Revision Data](https://www.m3ter.com/docs/api#tag/LookupTableRevisionData)
        section - then you won't be able to change the Revision's field schema and
        you'll receive an error if you try do this. Create a new Revision instead, or
        delete the data items first.

        Args:
          fields: The list of fields of the Lookup Table Revision.

          keys: The ordered keys of the Lookup Table Revision.

          name: Descriptive name for the Lookup Table Revision.

          custom_fields: User defined fields enabling you to attach custom data. The value for a custom
              field can be either a string or a number.

              If `customFields` can also be defined for this entity at the Organizational
              level, `customField` values defined at individual level override values of
              `customFields` with the same name defined at Organization level.

              See
              [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
              in the m3ter documentation for more information.

          start_date: The start date of the Lookup Table Revision.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not lookup_table_id:
            raise ValueError(f"Expected a non-empty value for `lookup_table_id` but received {lookup_table_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._put(
            f"/organizations/{org_id}/lookuptables/{lookup_table_id}/revisions/{id}",
            body=await async_maybe_transform(
                {
                    "fields": fields,
                    "keys": keys,
                    "name": name,
                    "custom_fields": custom_fields,
                    "start_date": start_date,
                    "version": version,
                },
                lookup_table_revision_update_params.LookupTableRevisionUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LookupTableRevisionResponse,
        )

    def list(
        self,
        lookup_table_id: str,
        *,
        org_id: str | None = None,
        ids: SequenceNotStr[str] | Omit = omit,
        next_token: str | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[LookupTableRevisionResponse, AsyncCursor[LookupTableRevisionResponse]]:
        """
        List LookupTableRevision entities

        Args:
          ids: List of Lookup Table Revision IDs to retrieve

          next_token: Token to supply for multi page retrievals

          page_size: Number of LookupTable to retrieve per page

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not lookup_table_id:
            raise ValueError(f"Expected a non-empty value for `lookup_table_id` but received {lookup_table_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/lookuptables/{lookup_table_id}/revisions",
            page=AsyncCursor[LookupTableRevisionResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "ids": ids,
                        "next_token": next_token,
                        "page_size": page_size,
                    },
                    lookup_table_revision_list_params.LookupTableRevisionListParams,
                ),
            ),
            model=LookupTableRevisionResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        lookup_table_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LookupTableRevisionResponse:
        """
        Delete the Lookup Table Revision for the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not lookup_table_id:
            raise ValueError(f"Expected a non-empty value for `lookup_table_id` but received {lookup_table_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/organizations/{org_id}/lookuptables/{lookup_table_id}/revisions/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LookupTableRevisionResponse,
        )

    async def update_status(
        self,
        id: str,
        *,
        org_id: str | None = None,
        lookup_table_id: str,
        status: Literal["DRAFT", "PUBLISHED", "ARCHIVED"] | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> LookupTableRevisionResponse:
        """
        Update the status of a Lookup Table Revision for the given UUID.

        Args:
          status: Status of a Lookup Table Revision

          version: The version of the LookupTableRevision.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not lookup_table_id:
            raise ValueError(f"Expected a non-empty value for `lookup_table_id` but received {lookup_table_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._put(
            f"/organizations/{org_id}/lookuptables/{lookup_table_id}/revisions/{id}/status",
            body=await async_maybe_transform(
                {
                    "status": status,
                    "version": version,
                },
                lookup_table_revision_update_status_params.LookupTableRevisionUpdateStatusParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LookupTableRevisionResponse,
        )


class LookupTableRevisionsResourceWithRawResponse:
    def __init__(self, lookup_table_revisions: LookupTableRevisionsResource) -> None:
        self._lookup_table_revisions = lookup_table_revisions

        self.create = to_raw_response_wrapper(
            lookup_table_revisions.create,
        )
        self.retrieve = to_raw_response_wrapper(
            lookup_table_revisions.retrieve,
        )
        self.update = to_raw_response_wrapper(
            lookup_table_revisions.update,
        )
        self.list = to_raw_response_wrapper(
            lookup_table_revisions.list,
        )
        self.delete = to_raw_response_wrapper(
            lookup_table_revisions.delete,
        )
        self.update_status = to_raw_response_wrapper(
            lookup_table_revisions.update_status,
        )


class AsyncLookupTableRevisionsResourceWithRawResponse:
    def __init__(self, lookup_table_revisions: AsyncLookupTableRevisionsResource) -> None:
        self._lookup_table_revisions = lookup_table_revisions

        self.create = async_to_raw_response_wrapper(
            lookup_table_revisions.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            lookup_table_revisions.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            lookup_table_revisions.update,
        )
        self.list = async_to_raw_response_wrapper(
            lookup_table_revisions.list,
        )
        self.delete = async_to_raw_response_wrapper(
            lookup_table_revisions.delete,
        )
        self.update_status = async_to_raw_response_wrapper(
            lookup_table_revisions.update_status,
        )


class LookupTableRevisionsResourceWithStreamingResponse:
    def __init__(self, lookup_table_revisions: LookupTableRevisionsResource) -> None:
        self._lookup_table_revisions = lookup_table_revisions

        self.create = to_streamed_response_wrapper(
            lookup_table_revisions.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            lookup_table_revisions.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            lookup_table_revisions.update,
        )
        self.list = to_streamed_response_wrapper(
            lookup_table_revisions.list,
        )
        self.delete = to_streamed_response_wrapper(
            lookup_table_revisions.delete,
        )
        self.update_status = to_streamed_response_wrapper(
            lookup_table_revisions.update_status,
        )


class AsyncLookupTableRevisionsResourceWithStreamingResponse:
    def __init__(self, lookup_table_revisions: AsyncLookupTableRevisionsResource) -> None:
        self._lookup_table_revisions = lookup_table_revisions

        self.create = async_to_streamed_response_wrapper(
            lookup_table_revisions.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            lookup_table_revisions.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            lookup_table_revisions.update,
        )
        self.list = async_to_streamed_response_wrapper(
            lookup_table_revisions.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            lookup_table_revisions.delete,
        )
        self.update_status = async_to_streamed_response_wrapper(
            lookup_table_revisions.update_status,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from .._types import SequenceNotStr
from .._utils import PropertyInfo

__all__ = ["CreditReasonListParams"]


class CreditReasonListParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    archived: bool
    """TRUE / FALSE archived flag to filter the list.

    CreditReasons can be archived once they are obsolete.

    - TRUE includes archived CreditReasons.
    - FALSE excludes CreditReasons that are archived.
    """

    codes: SequenceNotStr[str]
    """List of Credit Reason short codes to retrieve."""

    ids: SequenceNotStr[str]
    """List of Credit Reason IDs to retrieve."""

    next_token: Annotated[str, PropertyInfo(alias="nextToken")]
    """`nextToken` for multi page retrievals."""

    page_size: Annotated[int, PropertyInfo(alias="pageSize")]
    """Number of credit reasons to retrieve per page."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .lookup_table_revision_data_job_list_params import (
    LookupTableRevisionDataJobListParams as LookupTableRevisionDataJobListParams,
)
from .lookup_table_revision_data_job_list_response import (
    LookupTableRevisionDataJobListResponse as LookupTableRevisionDataJobListResponse,
)
from .lookup_table_revision_data_job_delete_response import (
    LookupTableRevisionDataJobDeleteResponse as LookupTableRevisionDataJobDeleteResponse,
)
from .lookup_table_revision_data_job_download_params import (
    LookupTableRevisionDataJobDownloadParams as LookupTableRevisionDataJobDownloadParams,
)
from .lookup_table_revision_data_job_download_response import (
    LookupTableRevisionDataJobDownloadResponse as LookupTableRevisionDataJobDownloadResponse,
)
from .lookup_table_revision_data_job_retrieve_response import (
    LookupTableRevisionDataJobRetrieveResponse as LookupTableRevisionDataJobRetrieveResponse,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo
from .m3ter_signed_credentials_request_param import M3terSignedCredentialsRequestParam

__all__ = ["WebhookCreateParams"]


class WebhookCreateParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    credentials: Required[M3terSignedCredentialsRequestParam]
    """This schema defines the credentials required for m3ter request signing."""

    description: Required[str]

    name: Required[str]

    url: Required[str]
    """The URL to which the webhook requests will be sent."""

    active: bool

    code: str

    version: int
    """The version number of the entity:

    - **Create entity:** Not valid for initial insertion of new entity - _do not use
      for Create_. On initial Create, version is set at 1 and listed in the
      response.
    - **Update Entity:** On Update, version is required and must match the existing
      version because a check is performed to ensure sequential versioning is
      preserved. Version is incremented by 1 and listed in the response.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Union
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypedDict

from .._types import SequenceNotStr
from .._utils import PropertyInfo

__all__ = ["BalanceUpdateParams"]


class BalanceUpdateParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    account_id: Required[Annotated[str, PropertyInfo(alias="accountId")]]
    """The unique identifier (UUID) for the end customer Account."""

    code: Required[str]
    """Unique short code for the Balance."""

    currency: Required[str]
    """The currency code used for the Balance amount. For example: USD, GBP or EUR."""

    end_date: Required[Annotated[Union[str, datetime], PropertyInfo(alias="endDate", format="iso8601")]]
    """
    The date _(in ISO 8601 format)_ after which the Balance will no longer be active
    for the Account.

    **Note:** You can use the `rolloverEndDate` request parameter to define an
    extended grace period for continued draw-down against the Balance if any amount
    remains when the specified `endDate` is reached.
    """

    name: Required[str]
    """The official name for the Balance."""

    start_date: Required[Annotated[Union[str, datetime], PropertyInfo(alias="startDate", format="iso8601")]]
    """The date _(in ISO 8601 format)_ when the Balance becomes active."""

    allow_overdraft: Annotated[bool, PropertyInfo(alias="allowOverdraft")]
    """Allow balance amounts to fall below zero.

    This feature is enabled on request. Please get in touch with m3ter Support or
    your m3ter contact if you would like it enabling for your organization(s).
    """

    balance_draw_down_description: Annotated[str, PropertyInfo(alias="balanceDrawDownDescription")]
    """A description for the bill line items for draw-down charges against the Balance.

    _(Optional)._
    """

    consumptions_accounting_product_id: Annotated[str, PropertyInfo(alias="consumptionsAccountingProductId")]
    """
    Product ID that any Balance Consumed line items will be attributed to for
    accounting purposes.(_Optional_)
    """

    contract_id: Annotated[str, PropertyInfo(alias="contractId")]
    """
    The unique identifier (UUID) of a Contract on the Account that the Balance will
    be added to.
    """

    custom_fields: Annotated[Dict[str, Union[str, float]], PropertyInfo(alias="customFields")]
    """User defined fields enabling you to attach custom data.

    The value for a custom field can be either a string or a number.

    If `customFields` can also be defined for this entity at the Organizational
    level, `customField` values defined at individual level override values of
    `customFields` with the same name defined at Organization level.

    See
    [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
    in the m3ter documentation for more information.
    """

    description: str
    """A description of the Balance."""

    fees_accounting_product_id: Annotated[str, PropertyInfo(alias="feesAccountingProductId")]
    """
    Product ID that any Balance Fees line items will be attributed to for accounting
    purposes.(_Optional_)
    """

    line_item_types: Annotated[
        List[
            Literal[
                "STANDING_CHARGE",
                "USAGE",
                "MINIMUM_SPEND",
                "COUNTER_RUNNING_TOTAL_CHARGE",
                "COUNTER_ADJUSTMENT_DEBIT",
                "AD_HOC",
            ]
        ],
        PropertyInfo(alias="lineItemTypes"),
    ]
    """
    Specify the line item charge types that can draw-down at billing against the
    Balance amount. Options are:

    - `"MINIMUM_SPEND"`
    - `"STANDING_CHARGE"`
    - `"USAGE"`
    - `"COUNTER_RUNNING_TOTAL_CHARGE"`
    - `"COUNTER_ADJUSTMENT_DEBIT"`
    - `AD_HOC`

    **NOTE:** If no charge types are specified, by default _all types_ can draw-down
    against the Balance amount at billing.
    """

    overage_description: Annotated[str, PropertyInfo(alias="overageDescription")]
    """A description for Bill line items overage charges."""

    overage_surcharge_percent: Annotated[float, PropertyInfo(alias="overageSurchargePercent")]
    """
    Define a surcharge level, as a percentage of regular usage rating, applied to
    overages _(usage charges that exceed the Balance amount)_. For example, if the
    regular usage rate is $10 per unit of usage consumed and
    `overageSurchargePercent` is set at 10%, then any usage charged above the
    original Balance amount is charged at $11 per unit of usage.
    """

    product_ids: Annotated[SequenceNotStr[str], PropertyInfo(alias="productIds")]
    """
    Specify the Products whose consumption charges due at billing can be drawn-down
    against the Balance amount.

    **Note:** If you don't specify any Products for Balance draw-down, by default
    the consumption charges for any Product the Account consumes will be drawn-down
    against the Balance amount.
    """

    rollover_amount: Annotated[float, PropertyInfo(alias="rolloverAmount")]
    """
    The maximum amount that can be carried over past the Balance end date for
    draw-down at billing if there is any unused Balance amount when the end date is
    reached. Works with `rolloverEndDate` to define the amount and duration of a
    Balance "grace period". _(Optional)_

    **Notes:**

    - If you leave `rolloverAmount` empty and only enter a `rolloverEndDate`, any
      amount left over after the Balance end date is reached will be drawn-down
      against up to the specified `rolloverEndDate`.
    - You must enter a `rolloverEndDate`. If you only enter a `rolloverAmount`
      without entering a `rolloverEndDate`, you'll receive an error when trying to
      create or update the Balance.
    - If you don't want to grant any grace period for outstanding Balance amounts,
      then do not use `rolloverAmount` and `rolloverEndDate`.
    """

    rollover_end_date: Annotated[Union[str, datetime], PropertyInfo(alias="rolloverEndDate", format="iso8601")]
    """
    The end date _(in ISO 8601 format)_ for the grace period during which unused
    Balance amounts can be carried over and drawn-down against at billing.

    **Note:** Use `rolloverAmount` if you want to specify a maximum amount that can
    be carried over and made available for draw-down.
    """

    version: int
    """The version number of the entity:

    - **Create entity:** Not valid for initial insertion of new entity - _do not use
      for Create_. On initial Create, version is set at 1 and listed in the
      response.
    - **Update Entity:** On Update, version is required and must match the existing
      version because a check is performed to ensure sequential versioning is
      preserved. Version is incremented by 1 and listed in the response.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from .._types import SequenceNotStr
from .._utils import PropertyInfo

__all__ = ["AggregationListParams"]


class AggregationListParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    codes: SequenceNotStr[str]
    """List of Aggregation codes to retrieve.

    These are unique short codes to identify each Aggregation.
    """

    ids: SequenceNotStr[str]
    """List of Aggregation IDs to retrieve."""

    next_token: Annotated[str, PropertyInfo(alias="nextToken")]
    """`nextToken` for multi-page retrievals."""

    page_size: Annotated[int, PropertyInfo(alias="pageSize")]
    """Number of Aggregations to retrieve per page."""

    product_id: Annotated[SequenceNotStr[str], PropertyInfo(alias="productId")]
    """The UUIDs of the Products to retrieve Aggregations for."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncCursor, AsyncCursor
from ..._base_client import AsyncPaginator, make_request_options
from ...types.data_exports import job_list_params
from ...types.data_exports.data_export_job_response import DataExportJobResponse
from ...types.data_exports.job_get_download_url_response import JobGetDownloadURLResponse

__all__ = ["JobsResource", "AsyncJobsResource"]


class JobsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> JobsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return JobsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> JobsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return JobsResourceWithStreamingResponse(self)

    def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DataExportJobResponse:
        """
        Retrieve an Export Job for the given UUID.

        The response returns:

        - The source type for the data exported by the Export Job: one of USAGE or
          OPERATIONAL.
        - The status of the Export Job.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/organizations/{org_id}/dataexports/jobs/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DataExportJobResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        date_created_end: str | Omit = omit,
        date_created_start: str | Omit = omit,
        ids: SequenceNotStr[str] | Omit = omit,
        next_token: str | Omit = omit,
        page_size: int | Omit = omit,
        schedule_id: str | Omit = omit,
        status: Literal["PENDING", "RUNNING", "SUCCEEDED", "FAILED"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursor[DataExportJobResponse]:
        """
        Retrieve a list of Export Job entities.

        Args:
          date_created_end:
              Include only Job entities created before this date. Format:
              yyyy-MM-dd'T'HH:mm:ss'Z'

          date_created_start:
              Include only Job entities created on or after this date. Format:
              yyyy-MM-dd'T'HH:mm:ss'Z'

          ids: List Job entities for the given UUIDs

          next_token: nextToken for multi page retrievals

          page_size: Number of Jobs to retrieve per page

          schedule_id: List Job entities for the schedule UUID

          status: List Job entities for the status

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/dataexports/jobs",
            page=SyncCursor[DataExportJobResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "date_created_end": date_created_end,
                        "date_created_start": date_created_start,
                        "ids": ids,
                        "next_token": next_token,
                        "page_size": page_size,
                        "schedule_id": schedule_id,
                        "status": status,
                    },
                    job_list_params.JobListParams,
                ),
            ),
            model=DataExportJobResponse,
        )

    def get_download_url(
        self,
        job_id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> JobGetDownloadURLResponse:
        """
        Returns a presigned download URL for data export file download based on the
        `jobId` provided.

        If you omit `destinationIds` when setting up your
        [Ad-Hoc data exports](https://www.m3ter.com/docs/api#tag/ExportAdHoc) or
        [Scheduled data exports](https://www.m3ter.com/docs/api#tag/ExportSchedule),
        then the data is not copied to a destination but is available for you to
        download using the returned download URL.

        **Constraints:**

        - Only valid for Export jobs ran in the past 24 hours.
        - The download URL is time-bound and is only valid for 15 minutes.

        **NOTE!** This ExportDestination endpoint is available in Beta release version.
        See
        [Feature Release Stages](https://www.m3ter.com/docs/guides/getting-started/feature-release-stages)
        for Beta release definition.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not job_id:
            raise ValueError(f"Expected a non-empty value for `job_id` but received {job_id!r}")
        return self._get(
            f"/organizations/{org_id}/dataexports/jobs/{job_id}/getdownloadurl",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=JobGetDownloadURLResponse,
        )


class AsyncJobsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncJobsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncJobsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncJobsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return AsyncJobsResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DataExportJobResponse:
        """
        Retrieve an Export Job for the given UUID.

        The response returns:

        - The source type for the data exported by the Export Job: one of USAGE or
          OPERATIONAL.
        - The status of the Export Job.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/organizations/{org_id}/dataexports/jobs/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DataExportJobResponse,
        )

    def list(
        self,
        *,
        org_id: str | None = None,
        date_created_end: str | Omit = omit,
        date_created_start: str | Omit = omit,
        ids: SequenceNotStr[str] | Omit = omit,
        next_token: str | Omit = omit,
        page_size: int | Omit = omit,
        schedule_id: str | Omit = omit,
        status: Literal["PENDING", "RUNNING", "SUCCEEDED", "FAILED"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[DataExportJobResponse, AsyncCursor[DataExportJobResponse]]:
        """
        Retrieve a list of Export Job entities.

        Args:
          date_created_end:
              Include only Job entities created before this date. Format:
              yyyy-MM-dd'T'HH:mm:ss'Z'

          date_created_start:
              Include only Job entities created on or after this date. Format:
              yyyy-MM-dd'T'HH:mm:ss'Z'

          ids: List Job entities for the given UUIDs

          next_token: nextToken for multi page retrievals

          page_size: Number of Jobs to retrieve per page

          schedule_id: List Job entities for the schedule UUID

          status: List Job entities for the status

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/dataexports/jobs",
            page=AsyncCursor[DataExportJobResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "date_created_end": date_created_end,
                        "date_created_start": date_created_start,
                        "ids": ids,
                        "next_token": next_token,
                        "page_size": page_size,
                        "schedule_id": schedule_id,
                        "status": status,
                    },
                    job_list_params.JobListParams,
                ),
            ),
            model=DataExportJobResponse,
        )

    async def get_download_url(
        self,
        job_id: str,
        *,
        org_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> JobGetDownloadURLResponse:
        """
        Returns a presigned download URL for data export file download based on the
        `jobId` provided.

        If you omit `destinationIds` when setting up your
        [Ad-Hoc data exports](https://www.m3ter.com/docs/api#tag/ExportAdHoc) or
        [Scheduled data exports](https://www.m3ter.com/docs/api#tag/ExportSchedule),
        then the data is not copied to a destination but is available for you to
        download using the returned download URL.

        **Constraints:**

        - Only valid for Export jobs ran in the past 24 hours.
        - The download URL is time-bound and is only valid for 15 minutes.

        **NOTE!** This ExportDestination endpoint is available in Beta release version.
        See
        [Feature Release Stages](https://www.m3ter.com/docs/guides/getting-started/feature-release-stages)
        for Beta release definition.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not job_id:
            raise ValueError(f"Expected a non-empty value for `job_id` but received {job_id!r}")
        return await self._get(
            f"/organizations/{org_id}/dataexports/jobs/{job_id}/getdownloadurl",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=JobGetDownloadURLResponse,
        )


class JobsResourceWithRawResponse:
    def __init__(self, jobs: JobsResource) -> None:
        self._jobs = jobs

        self.retrieve = to_raw_response_wrapper(
            jobs.retrieve,
        )
        self.list = to_raw_response_wrapper(
            jobs.list,
        )
        self.get_download_url = to_raw_response_wrapper(
            jobs.get_download_url,
        )


class AsyncJobsResourceWithRawResponse:
    def __init__(self, jobs: AsyncJobsResource) -> None:
        self._jobs = jobs

        self.retrieve = async_to_raw_response_wrapper(
            jobs.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            jobs.list,
        )
        self.get_download_url = async_to_raw_response_wrapper(
            jobs.get_download_url,
        )


class JobsResourceWithStreamingResponse:
    def __init__(self, jobs: JobsResource) -> None:
        self._jobs = jobs

        self.retrieve = to_streamed_response_wrapper(
            jobs.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            jobs.list,
        )
        self.get_download_url = to_streamed_response_wrapper(
            jobs.get_download_url,
        )


class AsyncJobsResourceWithStreamingResponse:
    def __init__(self, jobs: AsyncJobsResource) -> None:
        self._jobs = jobs

        self.retrieve = async_to_streamed_response_wrapper(
            jobs.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            jobs.list,
        )
        self.get_download_url = async_to_streamed_response_wrapper(
            jobs.get_download_url,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["ChargeUpdateParams"]


class ChargeUpdateParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    account_id: Required[Annotated[str, PropertyInfo(alias="accountId")]]
    """The ID of the Account the Charge is being created for."""

    code: Required[str]
    """Unique short code for the Charge."""

    currency: Required[str]
    """Charge currency."""

    entity_type: Required[Annotated[Literal["AD_HOC", "BALANCE"], PropertyInfo(alias="entityType")]]
    """The entity type the Charge has been created for."""

    line_item_type: Required[Annotated[Literal["BALANCE_FEE", "AD_HOC"], PropertyInfo(alias="lineItemType")]]
    """Available line item types that can be used for billing a Charge."""

    name: Required[str]
    """Name of the Charge. Added to the Bill line item description for this Charge."""

    service_period_end_date: Required[
        Annotated[Union[str, datetime], PropertyInfo(alias="servicePeriodEndDate", format="iso8601")]
    ]
    """The service period end date (_in ISO-8601 format_)for the Charge.

    **NOTE:** End date is exclusive.
    """

    service_period_start_date: Required[
        Annotated[Union[str, datetime], PropertyInfo(alias="servicePeriodStartDate", format="iso8601")]
    ]
    """The service period start date (_in ISO-8601 format_) for the Charge."""

    accounting_product_id: Annotated[str, PropertyInfo(alias="accountingProductId")]
    """The Accounting Product ID assigned to the Charge."""

    amount: float
    """Amount of the Charge.

    If `amount` is provided, then `units` and `unitPrice` must be omitted.
    """

    bill_date: Annotated[str, PropertyInfo(alias="billDate")]
    """The date when the Charge will be added to a Bill."""

    contract_id: Annotated[str, PropertyInfo(alias="contractId")]
    """The ID of a Contract on the Account that the Charge will be added to."""

    description: str
    """The description added to the Bill line item for the Charge."""

    entity_id: Annotated[str, PropertyInfo(alias="entityId")]
    """The ID of the Charge linked entity.

    For example, the ID of an Account Balance if a Balance Charge.

    **NOTE:** If `entityType` is `BALANCE`, you must provide the `entityId` of the
    Balance the Charge is for.
    """

    notes: str
    """
    Used to enter information about the Charge for accounting purposes, such as the
    reason it was created. This information will not be added to a Bill line item
    for the Charge.
    """

    unit_price: Annotated[float, PropertyInfo(alias="unitPrice")]
    """Unit price.

    If `amount` is omitted, then provide together with `units`. When `amount` is
    provided, `unitPrice` must be omitted.
    """

    units: float
    """Number of units of the Charge.

    If `amount` is omitted, then provide together with `unitPrice`. When `amount` is
    provided, `units` must be omitted.
    """

    version: int
    """The version number of the entity:

    - **Create entity:** Not valid for initial insertion of new entity - _do not use
      for Create_. On initial Create, version is set at 1 and listed in the
      response.
    - **Update Entity:** On Update, version is required and must match the existing
      version because a check is performed to ensure sequential versioning is
      preserved. Version is incremented by 1 and listed in the response.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union
from datetime import date, datetime
from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncCursor, AsyncCursor
from ..._base_client import AsyncPaginator, make_request_options
from ...types.balances import (
    charge_schedule_list_params,
    charge_schedule_create_params,
    charge_schedule_update_params,
    charge_schedule_preview_params,
)
from ...types.balances.charge_schedule_list_response import ChargeScheduleListResponse
from ...types.balances.charge_schedule_create_response import ChargeScheduleCreateResponse
from ...types.balances.charge_schedule_delete_response import ChargeScheduleDeleteResponse
from ...types.balances.charge_schedule_update_response import ChargeScheduleUpdateResponse
from ...types.balances.charge_schedule_preview_response import ChargeSchedulePreviewResponse
from ...types.balances.charge_schedule_retrieve_response import ChargeScheduleRetrieveResponse

__all__ = ["ChargeSchedulesResource", "AsyncChargeSchedulesResource"]


class ChargeSchedulesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ChargeSchedulesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return ChargeSchedulesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ChargeSchedulesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return ChargeSchedulesResourceWithStreamingResponse(self)

    def create(
        self,
        balance_id: str,
        *,
        org_id: str | None = None,
        bill_frequency: Literal["DAILY", "WEEKLY", "MONTHLY", "ANNUALLY"],
        bill_frequency_interval: int,
        bill_in_advance: bool,
        charge_description: str,
        code: str,
        currency: str,
        name: str,
        service_period_end_date: Union[str, datetime],
        service_period_start_date: Union[str, datetime],
        amount: float | Omit = omit,
        bill_epoch: Union[str, date] | Omit = omit,
        custom_fields: Dict[str, Union[str, float]] | Omit = omit,
        unit_price: float | Omit = omit,
        units: float | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ChargeScheduleCreateResponse:
        """
        Create a new BalanceChargeSchedule.

        Args:
          bill_frequency: Represents standard scheduling frequencies options for a job.

          bill_frequency_interval: How often Bills are issued. For example, if billFrequency is `MONTHLY` and
              `billFrequencyInterval` is 3, Bills are issued every three months.

          bill_in_advance: Used to specify how Charges created by the Balance Charge Schedule are billed -
              either in arrears or in advance:

              - Use `false` for billing in arrears.
              - Use `true` for billing in advance.

          charge_description: The description for Charges created by the Balance Charge Schedule. Used on
              Bills for Charge line items.

          code: Unique short code for the Balance Charge Schedule.

          currency: The currency of the Charges created by the Balance Charge Schedule.

          name: The name of the Balance Charge Schedule.

          service_period_end_date: The service period end date (_in ISO-8601 format_) of the Balance Charge
              Schedule.

          service_period_start_date: The service period start date (_in ISO-8601 format)_ of the Balance Charge
              Schedule.

          amount: The amount of each Charge created by the Balance Charge Schedule. Must be
              omitted if `units` and `unitPrice` are provided.

          bill_epoch: Specify a billing cycle date (_in ISO-8601 format_) for when the first Bill is
              created for Balance Charges created by the Schedule, and also acts as a
              reference for when in the Schedule period subsequent Bills are created for the
              defined `billFrequency`. If left blank, then the relevant Epoch date from your
              Organization's configuration will be used as the billing cycle date instead.

          custom_fields: User defined fields enabling you to attach custom data. The value for a custom
              field can be either a string or a number.

              If `customFields` can also be defined for this entity at the Organizational
              level, `customField` values defined at individual level override values of
              `customFields` with the same name defined at Organization level.

              See
              [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
              in the m3ter documentation for more information.

          unit_price: Unit price for Charge. Must be provided when `units` is used. Must be omitted
              when `amount` is used.

          units: Number of units defined for the Charges created by the Schedule. Required when
              `unitPrice` is provided. Must be omitted when `amount` is used.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not balance_id:
            raise ValueError(f"Expected a non-empty value for `balance_id` but received {balance_id!r}")
        return self._post(
            f"/organizations/{org_id}/balances/{balance_id}/balancechargeschedules",
            body=maybe_transform(
                {
                    "bill_frequency": bill_frequency,
                    "bill_frequency_interval": bill_frequency_interval,
                    "bill_in_advance": bill_in_advance,
                    "charge_description": charge_description,
                    "code": code,
                    "currency": currency,
                    "name": name,
                    "service_period_end_date": service_period_end_date,
                    "service_period_start_date": service_period_start_date,
                    "amount": amount,
                    "bill_epoch": bill_epoch,
                    "custom_fields": custom_fields,
                    "unit_price": unit_price,
                    "units": units,
                    "version": version,
                },
                charge_schedule_create_params.ChargeScheduleCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ChargeScheduleCreateResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        balance_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ChargeScheduleRetrieveResponse:
        """
        Retrieve a BalanceChargeSchedule for the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not balance_id:
            raise ValueError(f"Expected a non-empty value for `balance_id` but received {balance_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/organizations/{org_id}/balances/{balance_id}/balancechargeschedules/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ChargeScheduleRetrieveResponse,
        )

    def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        balance_id: str,
        bill_frequency: Literal["DAILY", "WEEKLY", "MONTHLY", "ANNUALLY"],
        bill_frequency_interval: int,
        bill_in_advance: bool,
        charge_description: str,
        code: str,
        currency: str,
        name: str,
        service_period_end_date: Union[str, datetime],
        service_period_start_date: Union[str, datetime],
        amount: float | Omit = omit,
        bill_epoch: Union[str, date] | Omit = omit,
        custom_fields: Dict[str, Union[str, float]] | Omit = omit,
        unit_price: float | Omit = omit,
        units: float | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ChargeScheduleUpdateResponse:
        """
        Update a BalanceChargeSchedule for the given UUID.

        Args:
          bill_frequency: Represents standard scheduling frequencies options for a job.

          bill_frequency_interval: How often Bills are issued. For example, if billFrequency is `MONTHLY` and
              `billFrequencyInterval` is 3, Bills are issued every three months.

          bill_in_advance: Used to specify how Charges created by the Balance Charge Schedule are billed -
              either in arrears or in advance:

              - Use `false` for billing in arrears.
              - Use `true` for billing in advance.

          charge_description: The description for Charges created by the Balance Charge Schedule. Used on
              Bills for Charge line items.

          code: Unique short code for the Balance Charge Schedule.

          currency: The currency of the Charges created by the Balance Charge Schedule.

          name: The name of the Balance Charge Schedule.

          service_period_end_date: The service period end date (_in ISO-8601 format_) of the Balance Charge
              Schedule.

          service_period_start_date: The service period start date (_in ISO-8601 format)_ of the Balance Charge
              Schedule.

          amount: The amount of each Charge created by the Balance Charge Schedule. Must be
              omitted if `units` and `unitPrice` are provided.

          bill_epoch: Specify a billing cycle date (_in ISO-8601 format_) for when the first Bill is
              created for Balance Charges created by the Schedule, and also acts as a
              reference for when in the Schedule period subsequent Bills are created for the
              defined `billFrequency`. If left blank, then the relevant Epoch date from your
              Organization's configuration will be used as the billing cycle date instead.

          custom_fields: User defined fields enabling you to attach custom data. The value for a custom
              field can be either a string or a number.

              If `customFields` can also be defined for this entity at the Organizational
              level, `customField` values defined at individual level override values of
              `customFields` with the same name defined at Organization level.

              See
              [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
              in the m3ter documentation for more information.

          unit_price: Unit price for Charge. Must be provided when `units` is used. Must be omitted
              when `amount` is used.

          units: Number of units defined for the Charges created by the Schedule. Required when
              `unitPrice` is provided. Must be omitted when `amount` is used.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not balance_id:
            raise ValueError(f"Expected a non-empty value for `balance_id` but received {balance_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._put(
            f"/organizations/{org_id}/balances/{balance_id}/balancechargeschedules/{id}",
            body=maybe_transform(
                {
                    "bill_frequency": bill_frequency,
                    "bill_frequency_interval": bill_frequency_interval,
                    "bill_in_advance": bill_in_advance,
                    "charge_description": charge_description,
                    "code": code,
                    "currency": currency,
                    "name": name,
                    "service_period_end_date": service_period_end_date,
                    "service_period_start_date": service_period_start_date,
                    "amount": amount,
                    "bill_epoch": bill_epoch,
                    "custom_fields": custom_fields,
                    "unit_price": unit_price,
                    "units": units,
                    "version": version,
                },
                charge_schedule_update_params.ChargeScheduleUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ChargeScheduleUpdateResponse,
        )

    def list(
        self,
        balance_id: str,
        *,
        org_id: str | None = None,
        ids: SequenceNotStr[str] | Omit = omit,
        next_token: str | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursor[ChargeScheduleListResponse]:
        """
        Retrieve a list of BalanceChargeSchedule entities

        Args:
          next_token: nextToken for multi page retrievals

          page_size: Number of BalanceChargeSchedules to retrieve per page

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not balance_id:
            raise ValueError(f"Expected a non-empty value for `balance_id` but received {balance_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/balances/{balance_id}/balancechargeschedules",
            page=SyncCursor[ChargeScheduleListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "ids": ids,
                        "next_token": next_token,
                        "page_size": page_size,
                    },
                    charge_schedule_list_params.ChargeScheduleListParams,
                ),
            ),
            model=ChargeScheduleListResponse,
        )

    def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        balance_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ChargeScheduleDeleteResponse:
        """
        Delete the BalanceChargeSchedule for the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not balance_id:
            raise ValueError(f"Expected a non-empty value for `balance_id` but received {balance_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/organizations/{org_id}/balances/{balance_id}/balancechargeschedules/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ChargeScheduleDeleteResponse,
        )

    def preview(
        self,
        balance_id: str,
        *,
        org_id: str | None = None,
        bill_frequency: Literal["DAILY", "WEEKLY", "MONTHLY", "ANNUALLY"],
        bill_frequency_interval: int,
        bill_in_advance: bool,
        charge_description: str,
        code: str,
        currency: str,
        name: str,
        service_period_end_date: Union[str, datetime],
        service_period_start_date: Union[str, datetime],
        next_token: str | Omit = omit,
        page_size: int | Omit = omit,
        amount: float | Omit = omit,
        bill_epoch: Union[str, date] | Omit = omit,
        custom_fields: Dict[str, Union[str, float]] | Omit = omit,
        unit_price: float | Omit = omit,
        units: float | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ChargeSchedulePreviewResponse:
        """Previews the Charges this Schedule would create, without persisting them.

        You
        can use this call to obtain a preview of the Charges a Schedule you plan to
        create for a Balance would generate.

        Args:
          bill_frequency: Represents standard scheduling frequencies options for a job.

          bill_frequency_interval: How often Bills are issued. For example, if billFrequency is `MONTHLY` and
              `billFrequencyInterval` is 3, Bills are issued every three months.

          bill_in_advance: Used to specify how Charges created by the Balance Charge Schedule are billed -
              either in arrears or in advance:

              - Use `false` for billing in arrears.
              - Use `true` for billing in advance.

          charge_description: The description for Charges created by the Balance Charge Schedule. Used on
              Bills for Charge line items.

          code: Unique short code for the Balance Charge Schedule.

          currency: The currency of the Charges created by the Balance Charge Schedule.

          name: The name of the Balance Charge Schedule.

          service_period_end_date: The service period end date (_in ISO-8601 format_) of the Balance Charge
              Schedule.

          service_period_start_date: The service period start date (_in ISO-8601 format)_ of the Balance Charge
              Schedule.

          next_token: nextToken for multi page retrievals

          page_size: Number of Charges to retrieve per page

          amount: The amount of each Charge created by the Balance Charge Schedule. Must be
              omitted if `units` and `unitPrice` are provided.

          bill_epoch: Specify a billing cycle date (_in ISO-8601 format_) for when the first Bill is
              created for Balance Charges created by the Schedule, and also acts as a
              reference for when in the Schedule period subsequent Bills are created for the
              defined `billFrequency`. If left blank, then the relevant Epoch date from your
              Organization's configuration will be used as the billing cycle date instead.

          custom_fields: User defined fields enabling you to attach custom data. The value for a custom
              field can be either a string or a number.

              If `customFields` can also be defined for this entity at the Organizational
              level, `customField` values defined at individual level override values of
              `customFields` with the same name defined at Organization level.

              See
              [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
              in the m3ter documentation for more information.

          unit_price: Unit price for Charge. Must be provided when `units` is used. Must be omitted
              when `amount` is used.

          units: Number of units defined for the Charges created by the Schedule. Required when
              `unitPrice` is provided. Must be omitted when `amount` is used.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not balance_id:
            raise ValueError(f"Expected a non-empty value for `balance_id` but received {balance_id!r}")
        return self._post(
            f"/organizations/{org_id}/balances/{balance_id}/balancechargeschedules/preview",
            body=maybe_transform(
                {
                    "bill_frequency": bill_frequency,
                    "bill_frequency_interval": bill_frequency_interval,
                    "bill_in_advance": bill_in_advance,
                    "charge_description": charge_description,
                    "code": code,
                    "currency": currency,
                    "name": name,
                    "service_period_end_date": service_period_end_date,
                    "service_period_start_date": service_period_start_date,
                    "amount": amount,
                    "bill_epoch": bill_epoch,
                    "custom_fields": custom_fields,
                    "unit_price": unit_price,
                    "units": units,
                    "version": version,
                },
                charge_schedule_preview_params.ChargeSchedulePreviewParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "next_token": next_token,
                        "page_size": page_size,
                    },
                    charge_schedule_preview_params.ChargeSchedulePreviewParams,
                ),
            ),
            cast_to=ChargeSchedulePreviewResponse,
        )


class AsyncChargeSchedulesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncChargeSchedulesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncChargeSchedulesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncChargeSchedulesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return AsyncChargeSchedulesResourceWithStreamingResponse(self)

    async def create(
        self,
        balance_id: str,
        *,
        org_id: str | None = None,
        bill_frequency: Literal["DAILY", "WEEKLY", "MONTHLY", "ANNUALLY"],
        bill_frequency_interval: int,
        bill_in_advance: bool,
        charge_description: str,
        code: str,
        currency: str,
        name: str,
        service_period_end_date: Union[str, datetime],
        service_period_start_date: Union[str, datetime],
        amount: float | Omit = omit,
        bill_epoch: Union[str, date] | Omit = omit,
        custom_fields: Dict[str, Union[str, float]] | Omit = omit,
        unit_price: float | Omit = omit,
        units: float | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ChargeScheduleCreateResponse:
        """
        Create a new BalanceChargeSchedule.

        Args:
          bill_frequency: Represents standard scheduling frequencies options for a job.

          bill_frequency_interval: How often Bills are issued. For example, if billFrequency is `MONTHLY` and
              `billFrequencyInterval` is 3, Bills are issued every three months.

          bill_in_advance: Used to specify how Charges created by the Balance Charge Schedule are billed -
              either in arrears or in advance:

              - Use `false` for billing in arrears.
              - Use `true` for billing in advance.

          charge_description: The description for Charges created by the Balance Charge Schedule. Used on
              Bills for Charge line items.

          code: Unique short code for the Balance Charge Schedule.

          currency: The currency of the Charges created by the Balance Charge Schedule.

          name: The name of the Balance Charge Schedule.

          service_period_end_date: The service period end date (_in ISO-8601 format_) of the Balance Charge
              Schedule.

          service_period_start_date: The service period start date (_in ISO-8601 format)_ of the Balance Charge
              Schedule.

          amount: The amount of each Charge created by the Balance Charge Schedule. Must be
              omitted if `units` and `unitPrice` are provided.

          bill_epoch: Specify a billing cycle date (_in ISO-8601 format_) for when the first Bill is
              created for Balance Charges created by the Schedule, and also acts as a
              reference for when in the Schedule period subsequent Bills are created for the
              defined `billFrequency`. If left blank, then the relevant Epoch date from your
              Organization's configuration will be used as the billing cycle date instead.

          custom_fields: User defined fields enabling you to attach custom data. The value for a custom
              field can be either a string or a number.

              If `customFields` can also be defined for this entity at the Organizational
              level, `customField` values defined at individual level override values of
              `customFields` with the same name defined at Organization level.

              See
              [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
              in the m3ter documentation for more information.

          unit_price: Unit price for Charge. Must be provided when `units` is used. Must be omitted
              when `amount` is used.

          units: Number of units defined for the Charges created by the Schedule. Required when
              `unitPrice` is provided. Must be omitted when `amount` is used.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not balance_id:
            raise ValueError(f"Expected a non-empty value for `balance_id` but received {balance_id!r}")
        return await self._post(
            f"/organizations/{org_id}/balances/{balance_id}/balancechargeschedules",
            body=await async_maybe_transform(
                {
                    "bill_frequency": bill_frequency,
                    "bill_frequency_interval": bill_frequency_interval,
                    "bill_in_advance": bill_in_advance,
                    "charge_description": charge_description,
                    "code": code,
                    "currency": currency,
                    "name": name,
                    "service_period_end_date": service_period_end_date,
                    "service_period_start_date": service_period_start_date,
                    "amount": amount,
                    "bill_epoch": bill_epoch,
                    "custom_fields": custom_fields,
                    "unit_price": unit_price,
                    "units": units,
                    "version": version,
                },
                charge_schedule_create_params.ChargeScheduleCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ChargeScheduleCreateResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        org_id: str | None = None,
        balance_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ChargeScheduleRetrieveResponse:
        """
        Retrieve a BalanceChargeSchedule for the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not balance_id:
            raise ValueError(f"Expected a non-empty value for `balance_id` but received {balance_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/organizations/{org_id}/balances/{balance_id}/balancechargeschedules/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ChargeScheduleRetrieveResponse,
        )

    async def update(
        self,
        id: str,
        *,
        org_id: str | None = None,
        balance_id: str,
        bill_frequency: Literal["DAILY", "WEEKLY", "MONTHLY", "ANNUALLY"],
        bill_frequency_interval: int,
        bill_in_advance: bool,
        charge_description: str,
        code: str,
        currency: str,
        name: str,
        service_period_end_date: Union[str, datetime],
        service_period_start_date: Union[str, datetime],
        amount: float | Omit = omit,
        bill_epoch: Union[str, date] | Omit = omit,
        custom_fields: Dict[str, Union[str, float]] | Omit = omit,
        unit_price: float | Omit = omit,
        units: float | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ChargeScheduleUpdateResponse:
        """
        Update a BalanceChargeSchedule for the given UUID.

        Args:
          bill_frequency: Represents standard scheduling frequencies options for a job.

          bill_frequency_interval: How often Bills are issued. For example, if billFrequency is `MONTHLY` and
              `billFrequencyInterval` is 3, Bills are issued every three months.

          bill_in_advance: Used to specify how Charges created by the Balance Charge Schedule are billed -
              either in arrears or in advance:

              - Use `false` for billing in arrears.
              - Use `true` for billing in advance.

          charge_description: The description for Charges created by the Balance Charge Schedule. Used on
              Bills for Charge line items.

          code: Unique short code for the Balance Charge Schedule.

          currency: The currency of the Charges created by the Balance Charge Schedule.

          name: The name of the Balance Charge Schedule.

          service_period_end_date: The service period end date (_in ISO-8601 format_) of the Balance Charge
              Schedule.

          service_period_start_date: The service period start date (_in ISO-8601 format)_ of the Balance Charge
              Schedule.

          amount: The amount of each Charge created by the Balance Charge Schedule. Must be
              omitted if `units` and `unitPrice` are provided.

          bill_epoch: Specify a billing cycle date (_in ISO-8601 format_) for when the first Bill is
              created for Balance Charges created by the Schedule, and also acts as a
              reference for when in the Schedule period subsequent Bills are created for the
              defined `billFrequency`. If left blank, then the relevant Epoch date from your
              Organization's configuration will be used as the billing cycle date instead.

          custom_fields: User defined fields enabling you to attach custom data. The value for a custom
              field can be either a string or a number.

              If `customFields` can also be defined for this entity at the Organizational
              level, `customField` values defined at individual level override values of
              `customFields` with the same name defined at Organization level.

              See
              [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
              in the m3ter documentation for more information.

          unit_price: Unit price for Charge. Must be provided when `units` is used. Must be omitted
              when `amount` is used.

          units: Number of units defined for the Charges created by the Schedule. Required when
              `unitPrice` is provided. Must be omitted when `amount` is used.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not balance_id:
            raise ValueError(f"Expected a non-empty value for `balance_id` but received {balance_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._put(
            f"/organizations/{org_id}/balances/{balance_id}/balancechargeschedules/{id}",
            body=await async_maybe_transform(
                {
                    "bill_frequency": bill_frequency,
                    "bill_frequency_interval": bill_frequency_interval,
                    "bill_in_advance": bill_in_advance,
                    "charge_description": charge_description,
                    "code": code,
                    "currency": currency,
                    "name": name,
                    "service_period_end_date": service_period_end_date,
                    "service_period_start_date": service_period_start_date,
                    "amount": amount,
                    "bill_epoch": bill_epoch,
                    "custom_fields": custom_fields,
                    "unit_price": unit_price,
                    "units": units,
                    "version": version,
                },
                charge_schedule_update_params.ChargeScheduleUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ChargeScheduleUpdateResponse,
        )

    def list(
        self,
        balance_id: str,
        *,
        org_id: str | None = None,
        ids: SequenceNotStr[str] | Omit = omit,
        next_token: str | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[ChargeScheduleListResponse, AsyncCursor[ChargeScheduleListResponse]]:
        """
        Retrieve a list of BalanceChargeSchedule entities

        Args:
          next_token: nextToken for multi page retrievals

          page_size: Number of BalanceChargeSchedules to retrieve per page

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not balance_id:
            raise ValueError(f"Expected a non-empty value for `balance_id` but received {balance_id!r}")
        return self._get_api_list(
            f"/organizations/{org_id}/balances/{balance_id}/balancechargeschedules",
            page=AsyncCursor[ChargeScheduleListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "ids": ids,
                        "next_token": next_token,
                        "page_size": page_size,
                    },
                    charge_schedule_list_params.ChargeScheduleListParams,
                ),
            ),
            model=ChargeScheduleListResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        org_id: str | None = None,
        balance_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ChargeScheduleDeleteResponse:
        """
        Delete the BalanceChargeSchedule for the given UUID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not balance_id:
            raise ValueError(f"Expected a non-empty value for `balance_id` but received {balance_id!r}")
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/organizations/{org_id}/balances/{balance_id}/balancechargeschedules/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ChargeScheduleDeleteResponse,
        )

    async def preview(
        self,
        balance_id: str,
        *,
        org_id: str | None = None,
        bill_frequency: Literal["DAILY", "WEEKLY", "MONTHLY", "ANNUALLY"],
        bill_frequency_interval: int,
        bill_in_advance: bool,
        charge_description: str,
        code: str,
        currency: str,
        name: str,
        service_period_end_date: Union[str, datetime],
        service_period_start_date: Union[str, datetime],
        next_token: str | Omit = omit,
        page_size: int | Omit = omit,
        amount: float | Omit = omit,
        bill_epoch: Union[str, date] | Omit = omit,
        custom_fields: Dict[str, Union[str, float]] | Omit = omit,
        unit_price: float | Omit = omit,
        units: float | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ChargeSchedulePreviewResponse:
        """Previews the Charges this Schedule would create, without persisting them.

        You
        can use this call to obtain a preview of the Charges a Schedule you plan to
        create for a Balance would generate.

        Args:
          bill_frequency: Represents standard scheduling frequencies options for a job.

          bill_frequency_interval: How often Bills are issued. For example, if billFrequency is `MONTHLY` and
              `billFrequencyInterval` is 3, Bills are issued every three months.

          bill_in_advance: Used to specify how Charges created by the Balance Charge Schedule are billed -
              either in arrears or in advance:

              - Use `false` for billing in arrears.
              - Use `true` for billing in advance.

          charge_description: The description for Charges created by the Balance Charge Schedule. Used on
              Bills for Charge line items.

          code: Unique short code for the Balance Charge Schedule.

          currency: The currency of the Charges created by the Balance Charge Schedule.

          name: The name of the Balance Charge Schedule.

          service_period_end_date: The service period end date (_in ISO-8601 format_) of the Balance Charge
              Schedule.

          service_period_start_date: The service period start date (_in ISO-8601 format)_ of the Balance Charge
              Schedule.

          next_token: nextToken for multi page retrievals

          page_size: Number of Charges to retrieve per page

          amount: The amount of each Charge created by the Balance Charge Schedule. Must be
              omitted if `units` and `unitPrice` are provided.

          bill_epoch: Specify a billing cycle date (_in ISO-8601 format_) for when the first Bill is
              created for Balance Charges created by the Schedule, and also acts as a
              reference for when in the Schedule period subsequent Bills are created for the
              defined `billFrequency`. If left blank, then the relevant Epoch date from your
              Organization's configuration will be used as the billing cycle date instead.

          custom_fields: User defined fields enabling you to attach custom data. The value for a custom
              field can be either a string or a number.

              If `customFields` can also be defined for this entity at the Organizational
              level, `customField` values defined at individual level override values of
              `customFields` with the same name defined at Organization level.

              See
              [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
              in the m3ter documentation for more information.

          unit_price: Unit price for Charge. Must be provided when `units` is used. Must be omitted
              when `amount` is used.

          units: Number of units defined for the Charges created by the Schedule. Required when
              `unitPrice` is provided. Must be omitted when `amount` is used.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        if not balance_id:
            raise ValueError(f"Expected a non-empty value for `balance_id` but received {balance_id!r}")
        return await self._post(
            f"/organizations/{org_id}/balances/{balance_id}/balancechargeschedules/preview",
            body=await async_maybe_transform(
                {
                    "bill_frequency": bill_frequency,
                    "bill_frequency_interval": bill_frequency_interval,
                    "bill_in_advance": bill_in_advance,
                    "charge_description": charge_description,
                    "code": code,
                    "currency": currency,
                    "name": name,
                    "service_period_end_date": service_period_end_date,
                    "service_period_start_date": service_period_start_date,
                    "amount": amount,
                    "bill_epoch": bill_epoch,
                    "custom_fields": custom_fields,
                    "unit_price": unit_price,
                    "units": units,
                    "version": version,
                },
                charge_schedule_preview_params.ChargeSchedulePreviewParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "next_token": next_token,
                        "page_size": page_size,
                    },
                    charge_schedule_preview_params.ChargeSchedulePreviewParams,
                ),
            ),
            cast_to=ChargeSchedulePreviewResponse,
        )


class ChargeSchedulesResourceWithRawResponse:
    def __init__(self, charge_schedules: ChargeSchedulesResource) -> None:
        self._charge_schedules = charge_schedules

        self.create = to_raw_response_wrapper(
            charge_schedules.create,
        )
        self.retrieve = to_raw_response_wrapper(
            charge_schedules.retrieve,
        )
        self.update = to_raw_response_wrapper(
            charge_schedules.update,
        )
        self.list = to_raw_response_wrapper(
            charge_schedules.list,
        )
        self.delete = to_raw_response_wrapper(
            charge_schedules.delete,
        )
        self.preview = to_raw_response_wrapper(
            charge_schedules.preview,
        )


class AsyncChargeSchedulesResourceWithRawResponse:
    def __init__(self, charge_schedules: AsyncChargeSchedulesResource) -> None:
        self._charge_schedules = charge_schedules

        self.create = async_to_raw_response_wrapper(
            charge_schedules.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            charge_schedules.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            charge_schedules.update,
        )
        self.list = async_to_raw_response_wrapper(
            charge_schedules.list,
        )
        self.delete = async_to_raw_response_wrapper(
            charge_schedules.delete,
        )
        self.preview = async_to_raw_response_wrapper(
            charge_schedules.preview,
        )


class ChargeSchedulesResourceWithStreamingResponse:
    def __init__(self, charge_schedules: ChargeSchedulesResource) -> None:
        self._charge_schedules = charge_schedules

        self.create = to_streamed_response_wrapper(
            charge_schedules.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            charge_schedules.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            charge_schedules.update,
        )
        self.list = to_streamed_response_wrapper(
            charge_schedules.list,
        )
        self.delete = to_streamed_response_wrapper(
            charge_schedules.delete,
        )
        self.preview = to_streamed_response_wrapper(
            charge_schedules.preview,
        )


class AsyncChargeSchedulesResourceWithStreamingResponse:
    def __init__(self, charge_schedules: AsyncChargeSchedulesResource) -> None:
        self._charge_schedules = charge_schedules

        self.create = async_to_streamed_response_wrapper(
            charge_schedules.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            charge_schedules.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            charge_schedules.update,
        )
        self.list = async_to_streamed_response_wrapper(
            charge_schedules.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            charge_schedules.delete,
        )
        self.preview = async_to_streamed_response_wrapper(
            charge_schedules.preview,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Required, Annotated, TypedDict

from .._utils import PropertyInfo
from .measurement_request_param import MeasurementRequestParam

__all__ = ["UsageSubmitParams"]


class UsageSubmitParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    measurements: Required[Iterable[MeasurementRequestParam]]
    """Request containing the usage data measurements for submission."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["LookupTableRevisionUpdateStatusParams"]


class LookupTableRevisionUpdateStatusParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    lookup_table_id: Required[Annotated[str, PropertyInfo(alias="lookupTableId")]]

    status: Literal["DRAFT", "PUBLISHED", "ARCHIVED"]
    """Status of a Lookup Table Revision"""

    version: int
    """The version of the LookupTableRevision."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from m3ter import M3ter, AsyncM3ter
from tests.utils import assert_matches_type
from m3ter.pagination import SyncCursor, AsyncCursor
from m3ter.types.lookup_tables.lookup_table_revision_data import (
    LookupTableRevisionDataJobListResponse,
    LookupTableRevisionDataJobDeleteResponse,
    LookupTableRevisionDataJobDownloadResponse,
    LookupTableRevisionDataJobRetrieveResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestLookupTableRevisionDataJobs:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_retrieve(self, client: M3ter) -> None:
        lookup_table_revision_data_job = (
            client.lookup_tables.lookup_table_revision_data.lookup_table_revision_data_jobs.retrieve(
                id="id",
                lookup_table_id="lookupTableId",
                lookup_table_revision_id="lookupTableRevisionId",
            )
        )
        assert_matches_type(
            LookupTableRevisionDataJobRetrieveResponse, lookup_table_revision_data_job, path=["response"]
        )

    @parametrize
    def test_raw_response_retrieve(self, client: M3ter) -> None:
        response = (
            client.lookup_tables.lookup_table_revision_data.lookup_table_revision_data_jobs.with_raw_response.retrieve(
                id="id",
                lookup_table_id="lookupTableId",
                lookup_table_revision_id="lookupTableRevisionId",
            )
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        lookup_table_revision_data_job = response.parse()
        assert_matches_type(
            LookupTableRevisionDataJobRetrieveResponse, lookup_table_revision_data_job, path=["response"]
        )

    @parametrize
    def test_streaming_response_retrieve(self, client: M3ter) -> None:
        with client.lookup_tables.lookup_table_revision_data.lookup_table_revision_data_jobs.with_streaming_response.retrieve(
            id="id",
            lookup_table_id="lookupTableId",
            lookup_table_revision_id="lookupTableRevisionId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            lookup_table_revision_data_job = response.parse()
            assert_matches_type(
                LookupTableRevisionDataJobRetrieveResponse, lookup_table_revision_data_job, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `lookup_table_id` but received ''"):
            client.lookup_tables.lookup_table_revision_data.lookup_table_revision_data_jobs.with_raw_response.retrieve(
                id="id",
                lookup_table_id="",
                lookup_table_revision_id="lookupTableRevisionId",
            )

        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `lookup_table_revision_id` but received ''"
        ):
            client.lookup_tables.lookup_table_revision_data.lookup_table_revision_data_jobs.with_raw_response.retrieve(
                id="id",
                lookup_table_id="lookupTableId",
                lookup_table_revision_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.lookup_tables.lookup_table_revision_data.lookup_table_revision_data_jobs.with_raw_response.retrieve(
                id="",
                lookup_table_id="lookupTableId",
                lookup_table_revision_id="lookupTableRevisionId",
            )

    @parametrize
    def test_method_list(self, client: M3ter) -> None:
        lookup_table_revision_data_job = (
            client.lookup_tables.lookup_table_revision_data.lookup_table_revision_data_jobs.list(
                lookup_table_revision_id="lookupTableRevisionId",
                lookup_table_id="lookupTableId",
            )
        )
        assert_matches_type(
            SyncCursor[LookupTableRevisionDataJobListResponse], lookup_table_revision_data_job, path=["response"]
        )

    @parametrize
    def test_method_list_with_all_params(self, client: M3ter) -> None:
        lookup_table_revision_data_job = (
            client.lookup_tables.lookup_table_revision_data.lookup_table_revision_data_jobs.list(
                lookup_table_revision_id="lookupTableRevisionId",
                lookup_table_id="lookupTableId",
                next_token="nextToken",
                page_size=1,
            )
        )
        assert_matches_type(
            SyncCursor[LookupTableRevisionDataJobListResponse], lookup_table_revision_data_job, path=["response"]
        )

    @parametrize
    def test_raw_response_list(self, client: M3ter) -> None:
        response = (
            client.lookup_tables.lookup_table_revision_data.lookup_table_revision_data_jobs.with_raw_response.list(
                lookup_table_revision_id="lookupTableRevisionId",
                lookup_table_id="lookupTableId",
            )
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        lookup_table_revision_data_job = response.parse()
        assert_matches_type(
            SyncCursor[LookupTableRevisionDataJobListResponse], lookup_table_revision_data_job, path=["response"]
        )

    @parametrize
    def test_streaming_response_list(self, client: M3ter) -> None:
        with client.lookup_tables.lookup_table_revision_data.lookup_table_revision_data_jobs.with_streaming_response.list(
            lookup_table_revision_id="lookupTableRevisionId",
            lookup_table_id="lookupTableId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            lookup_table_revision_data_job = response.parse()
            assert_matches_type(
                SyncCursor[LookupTableRevisionDataJobListResponse], lookup_table_revision_data_job, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_list(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `lookup_table_id` but received ''"):
            client.lookup_tables.lookup_table_revision_data.lookup_table_revision_data_jobs.with_raw_response.list(
                lookup_table_revision_id="lookupTableRevisionId",
                lookup_table_id="",
            )

        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `lookup_table_revision_id` but received ''"
        ):
            client.lookup_tables.lookup_table_revision_data.lookup_table_revision_data_jobs.with_raw_response.list(
                lookup_table_revision_id="",
                lookup_table_id="lookupTableId",
            )

    @parametrize
    def test_method_delete(self, client: M3ter) -> None:
        lookup_table_revision_data_job = (
            client.lookup_tables.lookup_table_revision_data.lookup_table_revision_data_jobs.delete(
                id="id",
                lookup_table_id="lookupTableId",
                lookup_table_revision_id="lookupTableRevisionId",
            )
        )
        assert_matches_type(LookupTableRevisionDataJobDeleteResponse, lookup_table_revision_data_job, path=["response"])

    @parametrize
    def test_raw_response_delete(self, client: M3ter) -> None:
        response = (
            client.lookup_tables.lookup_table_revision_data.lookup_table_revision_data_jobs.with_raw_response.delete(
                id="id",
                lookup_table_id="lookupTableId",
                lookup_table_revision_id="lookupTableRevisionId",
            )
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        lookup_table_revision_data_job = response.parse()
        assert_matches_type(LookupTableRevisionDataJobDeleteResponse, lookup_table_revision_data_job, path=["response"])

    @parametrize
    def test_streaming_response_delete(self, client: M3ter) -> None:
        with client.lookup_tables.lookup_table_revision_data.lookup_table_revision_data_jobs.with_streaming_response.delete(
            id="id",
            lookup_table_id="lookupTableId",
            lookup_table_revision_id="lookupTableRevisionId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            lookup_table_revision_data_job = response.parse()
            assert_matches_type(
                LookupTableRevisionDataJobDeleteResponse, lookup_table_revision_data_job, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `lookup_table_id` but received ''"):
            client.lookup_tables.lookup_table_revision_data.lookup_table_revision_data_jobs.with_raw_response.delete(
                id="id",
                lookup_table_id="",
                lookup_table_revision_id="lookupTableRevisionId",
            )

        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `lookup_table_revision_id` but received ''"
        ):
            client.lookup_tables.lookup_table_revision_data.lookup_table_revision_data_jobs.with_raw_response.delete(
                id="id",
                lookup_table_id="lookupTableId",
                lookup_table_revision_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.lookup_tables.lookup_table_revision_data.lookup_table_revision_data_jobs.with_raw_response.delete(
                id="",
                lookup_table_id="lookupTableId",
                lookup_table_revision_id="lookupTableRevisionId",
            )

    @parametrize
    def test_method_download(self, client: M3ter) -> None:
        lookup_table_revision_data_job = (
            client.lookup_tables.lookup_table_revision_data.lookup_table_revision_data_jobs.download(
                lookup_table_revision_id="lookupTableRevisionId",
                lookup_table_id="lookupTableId",
                content_type="application/jsonl",
            )
        )
        assert_matches_type(
            LookupTableRevisionDataJobDownloadResponse, lookup_table_revision_data_job, path=["response"]
        )

    @parametrize
    def test_raw_response_download(self, client: M3ter) -> None:
        response = (
            client.lookup_tables.lookup_table_revision_data.lookup_table_revision_data_jobs.with_raw_response.download(
                lookup_table_revision_id="lookupTableRevisionId",
                lookup_table_id="lookupTableId",
                content_type="application/jsonl",
            )
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        lookup_table_revision_data_job = response.parse()
        assert_matches_type(
            LookupTableRevisionDataJobDownloadResponse, lookup_table_revision_data_job, path=["response"]
        )

    @parametrize
    def test_streaming_response_download(self, client: M3ter) -> None:
        with client.lookup_tables.lookup_table_revision_data.lookup_table_revision_data_jobs.with_streaming_response.download(
            lookup_table_revision_id="lookupTableRevisionId",
            lookup_table_id="lookupTableId",
            content_type="application/jsonl",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            lookup_table_revision_data_job = response.parse()
            assert_matches_type(
                LookupTableRevisionDataJobDownloadResponse, lookup_table_revision_data_job, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_download(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `lookup_table_id` but received ''"):
            client.lookup_tables.lookup_table_revision_data.lookup_table_revision_data_jobs.with_raw_response.download(
                lookup_table_revision_id="lookupTableRevisionId",
                lookup_table_id="",
                content_type="application/jsonl",
            )

        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `lookup_table_revision_id` but received ''"
        ):
            client.lookup_tables.lookup_table_revision_data.lookup_table_revision_data_jobs.with_raw_response.download(
                lookup_table_revision_id="",
                lookup_table_id="lookupTableId",
                content_type="application/jsonl",
            )


class TestAsyncLookupTableRevisionDataJobs:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncM3ter) -> None:
        lookup_table_revision_data_job = (
            await async_client.lookup_tables.lookup_table_revision_data.lookup_table_revision_data_jobs.retrieve(
                id="id",
                lookup_table_id="lookupTableId",
                lookup_table_revision_id="lookupTableRevisionId",
            )
        )
        assert_matches_type(
            LookupTableRevisionDataJobRetrieveResponse, lookup_table_revision_data_job, path=["response"]
        )

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncM3ter) -> None:
        response = await async_client.lookup_tables.lookup_table_revision_data.lookup_table_revision_data_jobs.with_raw_response.retrieve(
            id="id",
            lookup_table_id="lookupTableId",
            lookup_table_revision_id="lookupTableRevisionId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        lookup_table_revision_data_job = await response.parse()
        assert_matches_type(
            LookupTableRevisionDataJobRetrieveResponse, lookup_table_revision_data_job, path=["response"]
        )

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncM3ter) -> None:
        async with async_client.lookup_tables.lookup_table_revision_data.lookup_table_revision_data_jobs.with_streaming_response.retrieve(
            id="id",
            lookup_table_id="lookupTableId",
            lookup_table_revision_id="lookupTableRevisionId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            lookup_table_revision_data_job = await response.parse()
            assert_matches_type(
                LookupTableRevisionDataJobRetrieveResponse, lookup_table_revision_data_job, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `lookup_table_id` but received ''"):
            await async_client.lookup_tables.lookup_table_revision_data.lookup_table_revision_data_jobs.with_raw_response.retrieve(
                id="id",
                lookup_table_id="",
                lookup_table_revision_id="lookupTableRevisionId",
            )

        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `lookup_table_revision_id` but received ''"
        ):
            await async_client.lookup_tables.lookup_table_revision_data.lookup_table_revision_data_jobs.with_raw_response.retrieve(
                id="id",
                lookup_table_id="lookupTableId",
                lookup_table_revision_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.lookup_tables.lookup_table_revision_data.lookup_table_revision_data_jobs.with_raw_response.retrieve(
                id="",
                lookup_table_id="lookupTableId",
                lookup_table_revision_id="lookupTableRevisionId",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncM3ter) -> None:
        lookup_table_revision_data_job = (
            await async_client.lookup_tables.lookup_table_revision_data.lookup_table_revision_data_jobs.list(
                lookup_table_revision_id="lookupTableRevisionId",
                lookup_table_id="lookupTableId",
            )
        )
        assert_matches_type(
            AsyncCursor[LookupTableRevisionDataJobListResponse], lookup_table_revision_data_job, path=["response"]
        )

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncM3ter) -> None:
        lookup_table_revision_data_job = (
            await async_client.lookup_tables.lookup_table_revision_data.lookup_table_revision_data_jobs.list(
                lookup_table_revision_id="lookupTableRevisionId",
                lookup_table_id="lookupTableId",
                next_token="nextToken",
                page_size=1,
            )
        )
        assert_matches_type(
            AsyncCursor[LookupTableRevisionDataJobListResponse], lookup_table_revision_data_job, path=["response"]
        )

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncM3ter) -> None:
        response = await async_client.lookup_tables.lookup_table_revision_data.lookup_table_revision_data_jobs.with_raw_response.list(
            lookup_table_revision_id="lookupTableRevisionId",
            lookup_table_id="lookupTableId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        lookup_table_revision_data_job = await response.parse()
        assert_matches_type(
            AsyncCursor[LookupTableRevisionDataJobListResponse], lookup_table_revision_data_job, path=["response"]
        )

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncM3ter) -> None:
        async with async_client.lookup_tables.lookup_table_revision_data.lookup_table_revision_data_jobs.with_streaming_response.list(
            lookup_table_revision_id="lookupTableRevisionId",
            lookup_table_id="lookupTableId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            lookup_table_revision_data_job = await response.parse()
            assert_matches_type(
                AsyncCursor[LookupTableRevisionDataJobListResponse], lookup_table_revision_data_job, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_list(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `lookup_table_id` but received ''"):
            await async_client.lookup_tables.lookup_table_revision_data.lookup_table_revision_data_jobs.with_raw_response.list(
                lookup_table_revision_id="lookupTableRevisionId",
                lookup_table_id="",
            )

        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `lookup_table_revision_id` but received ''"
        ):
            await async_client.lookup_tables.lookup_table_revision_data.lookup_table_revision_data_jobs.with_raw_response.list(
                lookup_table_revision_id="",
                lookup_table_id="lookupTableId",
            )

    @parametrize
    async def test_method_delete(self, async_client: AsyncM3ter) -> None:
        lookup_table_revision_data_job = (
            await async_client.lookup_tables.lookup_table_revision_data.lookup_table_revision_data_jobs.delete(
                id="id",
                lookup_table_id="lookupTableId",
                lookup_table_revision_id="lookupTableRevisionId",
            )
        )
        assert_matches_type(LookupTableRevisionDataJobDeleteResponse, lookup_table_revision_data_job, path=["response"])

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncM3ter) -> None:
        response = await async_client.lookup_tables.lookup_table_revision_data.lookup_table_revision_data_jobs.with_raw_response.delete(
            id="id",
            lookup_table_id="lookupTableId",
            lookup_table_revision_id="lookupTableRevisionId",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        lookup_table_revision_data_job = await response.parse()
        assert_matches_type(LookupTableRevisionDataJobDeleteResponse, lookup_table_revision_data_job, path=["response"])

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncM3ter) -> None:
        async with async_client.lookup_tables.lookup_table_revision_data.lookup_table_revision_data_jobs.with_streaming_response.delete(
            id="id",
            lookup_table_id="lookupTableId",
            lookup_table_revision_id="lookupTableRevisionId",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            lookup_table_revision_data_job = await response.parse()
            assert_matches_type(
                LookupTableRevisionDataJobDeleteResponse, lookup_table_revision_data_job, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `lookup_table_id` but received ''"):
            await async_client.lookup_tables.lookup_table_revision_data.lookup_table_revision_data_jobs.with_raw_response.delete(
                id="id",
                lookup_table_id="",
                lookup_table_revision_id="lookupTableRevisionId",
            )

        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `lookup_table_revision_id` but received ''"
        ):
            await async_client.lookup_tables.lookup_table_revision_data.lookup_table_revision_data_jobs.with_raw_response.delete(
                id="id",
                lookup_table_id="lookupTableId",
                lookup_table_revision_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.lookup_tables.lookup_table_revision_data.lookup_table_revision_data_jobs.with_raw_response.delete(
                id="",
                lookup_table_id="lookupTableId",
                lookup_table_revision_id="lookupTableRevisionId",
            )

    @parametrize
    async def test_method_download(self, async_client: AsyncM3ter) -> None:
        lookup_table_revision_data_job = (
            await async_client.lookup_tables.lookup_table_revision_data.lookup_table_revision_data_jobs.download(
                lookup_table_revision_id="lookupTableRevisionId",
                lookup_table_id="lookupTableId",
                content_type="application/jsonl",
            )
        )
        assert_matches_type(
            LookupTableRevisionDataJobDownloadResponse, lookup_table_revision_data_job, path=["response"]
        )

    @parametrize
    async def test_raw_response_download(self, async_client: AsyncM3ter) -> None:
        response = await async_client.lookup_tables.lookup_table_revision_data.lookup_table_revision_data_jobs.with_raw_response.download(
            lookup_table_revision_id="lookupTableRevisionId",
            lookup_table_id="lookupTableId",
            content_type="application/jsonl",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        lookup_table_revision_data_job = await response.parse()
        assert_matches_type(
            LookupTableRevisionDataJobDownloadResponse, lookup_table_revision_data_job, path=["response"]
        )

    @parametrize
    async def test_streaming_response_download(self, async_client: AsyncM3ter) -> None:
        async with async_client.lookup_tables.lookup_table_revision_data.lookup_table_revision_data_jobs.with_streaming_response.download(
            lookup_table_revision_id="lookupTableRevisionId",
            lookup_table_id="lookupTableId",
            content_type="application/jsonl",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            lookup_table_revision_data_job = await response.parse()
            assert_matches_type(
                LookupTableRevisionDataJobDownloadResponse, lookup_table_revision_data_job, path=["response"]
            )

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_download(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `lookup_table_id` but received ''"):
            await async_client.lookup_tables.lookup_table_revision_data.lookup_table_revision_data_jobs.with_raw_response.download(
                lookup_table_revision_id="lookupTableRevisionId",
                lookup_table_id="",
                content_type="application/jsonl",
            )

        with pytest.raises(
            ValueError, match=r"Expected a non-empty value for `lookup_table_revision_id` but received ''"
        ):
            await async_client.lookup_tables.lookup_table_revision_data.lookup_table_revision_data_jobs.with_raw_response.download(
                lookup_table_revision_id="",
                lookup_table_id="lookupTableId",
                content_type="application/jsonl",
            )
